"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.json = exports.redirect = exports.notFound = void 0;
const response_utils_1 = require("./lib/response-utils");
/**
 * Send the user a 404
 */
function notFound(init = 404) {
    const body = { notFound: true };
    if (typeof init === 'number') {
        return { status: init, body };
    }
    return Object.assign(Object.assign({ status: 404 }, init), { body });
}
exports.notFound = notFound;
/**
 * Redirect the user to another page. Defaults to temporary redirect.
 */
function redirect(destination, init = 302) {
    let responseInit;
    if (typeof init === 'number') {
        responseInit = { status: init };
    }
    else if ('permanent' in init) {
        responseInit = { status: init.permanent ? 301 : 302 };
    }
    else {
        responseInit = Object.assign({ status: 302 }, init);
    }
    const statusCode = (responseInit.status || 302);
    const body = { redirect: { destination, statusCode } };
    const headers = (0, response_utils_1.mergeHeaders)(responseInit.headers, {
        location: destination,
    });
    return Object.assign(Object.assign({}, responseInit), { headers,
        body });
}
exports.redirect = redirect;
/**
 * Return the props used to render the page, or as api response
 */
function json(props, init = {}) {
    let responseInit;
    if (typeof init === 'number') {
        responseInit = { status: init };
    }
    else {
        responseInit = Object.assign({}, init);
    }
    const headers = (0, response_utils_1.mergeHeaders)(responseInit.headers, {
        'content-type': 'application/json; charset=utf-8',
    });
    return Object.assign(Object.assign({}, responseInit), { headers, body: { props } });
}
exports.json = json;
//# sourceMappingURL=responses.js.map