/// <reference types="node" />
import { IncomingMessage, ServerResponse } from 'http';
import { ParsedUrlQuery } from './querystring';
export declare type RedirectStatusCode = 301 | 302 | 303 | 307 | 308;
export declare type Redirect = {
    statusCode: RedirectStatusCode;
    destination: string;
    basePath?: false;
} | {
    permanent: boolean;
    destination: string;
    basePath?: false;
};
export declare type GetServerSidePropsContext<Q extends ParsedUrlQuery = ParsedUrlQuery> = {
    req: IncomingMessage & {
        cookies: Record<string, string>;
    };
    res: ServerResponse;
    params?: Q;
    query: ParsedUrlQuery;
};
export declare type PropResult<P extends Record<string, unknown>> = {
    props: P;
};
export declare type NotFoundResult = {
    notFound: true;
};
export declare type RedirectResult = {
    redirect: Redirect;
};
export declare type GetServerSidePropsResult<P extends Record<string, unknown>> = PropResult<P> | RedirectResult | NotFoundResult;
export declare type GetServerSideProps<P extends {
    [key: string]: any;
} = {
    [key: string]: any;
}, Q extends ParsedUrlQuery = ParsedUrlQuery> = (context: GetServerSidePropsContext<Q>) => Promise<GetServerSidePropsResult<P>>;
